@@ -1,4 +1,4 @@
-/* $Id: acconfig.h,v 1.3 2000/02/01 02:18:26 lukem Exp $ */
+/* $Id: acconfig.h,v 1.6 2000/09/17 23:29:12 lukem Exp $ */
 
 @TOP@
 @BOTTOM@
@@ -9,12 +9,27 @@
 /* Define if in_port_t exists */
 #undef HAVE_IN_PORT_T
 
+/* Define if struct sockaddr.sa_len exists (implies sockaddr_in.sin_len, etc) */
+#undef HAVE_SOCKADDR_SA_LEN
+
+/* Define if socklen_t exists */
+#undef HAVE_SOCKLEN_T
+
 /* Define if AF_INET6 exists in <sys/socket.h> */
 #undef HAVE_AF_INET6
 
 /* Define if `struct sockaddr_in6' exists in <netinet/in.h> */
 #undef HAVE_SOCKADDR_IN6
 
+/* Define if `struct addrinfo' exists in <netdb.h> */
+#undef HAVE_ADDRINFO
+
+/*
+ * Define if <netdb.h> contains AI_NUMERICHOST et al.
+ * Systems which only implement RFC2133 will need this.
+ */
+#undef HAVE_RFC2553_NETDB
+
 /* Define if `struct direct' has a d_namlen element */
 #undef HAVE_D_NAMLEN
 
@@ -27,6 +42,9 @@
 /* Define if fclose() is declared in <stdio.h> */
 #undef HAVE_FCLOSE_D
 
+/* Define if getpass() is declared in <stdlib.h> or <unistd.h> */
+#undef HAVE_GETPASS_D
+
 /* Define if optarg is declared in <stdlib.h> or <unistd.h> */
 #undef HAVE_OPTARG_D
 
