@@ -1,4 +1,4 @@
-/*	$NetBSD: prompt.c,v 1.6 1999/11/12 01:05:07 lukem Exp $	*/
+/*	$NetBSD: prompt.c,v 1.7 2000/09/04 22:06:31 lukem Exp $	*/
 
 /*-
  * Copyright (c) 1992, 1993
@@ -42,31 +42,32 @@
 #include "sys.h"
 #include "el.h"
 
-private char *prompt_default	__P((EditLine *));
-private char *prompt_default_r	__P((EditLine *));
+private char	*prompt_default(EditLine *);
+private char	*prompt_default_r(EditLine *);
 
 /* prompt_default():
  *	Just a default prompt, in case the user did not provide one
  */
 private char *
 /*ARGSUSED*/
-prompt_default(el)
-    EditLine *el;
+prompt_default(EditLine *el)
 {
-    static char a[3] = { '?', ' ', '\0' };
-    return a;
+	static char a[3] = {'?', ' ', '\0'};
+
+	return (a);
 }
 
+
 /* prompt_default_r():
  *	Just a default rprompt, in case the user did not provide one
  */
 private char *
 /*ARGSUSED*/
-prompt_default_r(el)
-    EditLine *el;
+prompt_default_r(EditLine *el)
 {
-    static char a[1] = { '\0' };
-    return a;
+	static char a[1] = {'\0'};
+
+	return (a);
 }
 
 
@@ -77,96 +78,87 @@ prompt_default_r(el)
  *	bit to flag them
  */
 protected void
-prompt_print(el, op)
-    EditLine *el;
-    int op;
+prompt_print(EditLine *el, int op)
 {
-    el_prompt_t	*elp;
-    char	*p;
+	el_prompt_t *elp;
+	char *p;
 
-    if (op == EL_PROMPT)
-	elp = &el->el_prompt;
-    else
-	elp = &el->el_rprompt;
-    p = (elp->p_func)(el);
-    while (*p)
-	re_putc(el, *p++);
+	if (op == EL_PROMPT)
+		elp = &el->el_prompt;
+	else
+		elp = &el->el_rprompt;
+	p = (elp->p_func) (el);
+	while (*p)
+		re_putc(el, *p++);
 
-    elp->p_pos.v = el->el_refresh.r_cursor.v;
-    elp->p_pos.h = el->el_refresh.r_cursor.h;
-} /* end prompt_print */
+	elp->p_pos.v = el->el_refresh.r_cursor.v;
+	elp->p_pos.h = el->el_refresh.r_cursor.h;
+}
 
 
 /* prompt_init():
  *	Initialize the prompt stuff
  */
 protected int
-prompt_init(el)
-    EditLine *el;
+prompt_init(EditLine *el)
 {
-    el->el_prompt.p_func = prompt_default;
-    el->el_prompt.p_pos.v = 0;
-    el->el_prompt.p_pos.h = 0;
-    el->el_rprompt.p_func = prompt_default_r;
-    el->el_rprompt.p_pos.v = 0;
-    el->el_rprompt.p_pos.h = 0;
-    return 0;
-} /* end prompt_init */
 
+	el->el_prompt.p_func = prompt_default;
+	el->el_prompt.p_pos.v = 0;
+	el->el_prompt.p_pos.h = 0;
+	el->el_rprompt.p_func = prompt_default_r;
+	el->el_rprompt.p_pos.v = 0;
+	el->el_rprompt.p_pos.h = 0;
+	return (0);
+}
 
 /* prompt_end():
  *	Clean up the prompt stuff
  */
 protected void
 /*ARGSUSED*/
-prompt_end(el)
-    EditLine *el;
+prompt_end(EditLine *el)
 {
-} /* end prompt_end */
+}
 
 
 /* prompt_set():
  *	Install a prompt printing function
  */
 protected int
-prompt_set(el, prf, op)
-    EditLine *el;
-    el_pfunc_t prf;
-    int op;
+prompt_set(EditLine *el, el_pfunc_t prf, int op)
 {
-    el_prompt_t *p;
+	el_prompt_t *p;
 
-    if (op == EL_PROMPT)
-	p = &el->el_prompt;
-    else
-	p = &el->el_rprompt;
-    if (prf == NULL) {
 	if (op == EL_PROMPT)
-	    p->p_func = prompt_default;
+		p = &el->el_prompt;
 	else
-	    p->p_func = prompt_default_r;
-    } else
-	p->p_func = prf;
-    p->p_pos.v = 0;
-    p->p_pos.h = 0;
-    return 0;
-} /* end prompt_set */
+		p = &el->el_rprompt;
+	if (prf == NULL) {
+		if (op == EL_PROMPT)
+			p->p_func = prompt_default;
+		else
+			p->p_func = prompt_default_r;
+	} else
+		p->p_func = prf;
+	p->p_pos.v = 0;
+	p->p_pos.h = 0;
+	return (0);
+}
 
 
 /* prompt_get():
  *	Retrieve the prompt printing function
  */
 protected int
-prompt_get(el, prf, op)
-    EditLine *el;
-    el_pfunc_t *prf;
-    int op;
+prompt_get(EditLine *el, el_pfunc_t *prf, int op)
 {
-    if (prf == NULL)
-	return -1;
-    if (op == EL_PROMPT)
-    	*prf = el->el_prompt.p_func;
-    else
-    	*prf = el->el_rprompt.p_func;
-    return 0;
-} /* end prompt_get */
+
+	if (prf == NULL)
+		return (-1);
+	if (op == EL_PROMPT)
+		*prf = el->el_prompt.p_func;
+	else
+		*prf = el->el_rprompt.p_func;
+	return (0);
+}
