@@ -1,4 +1,4 @@
-/*	$NetBSD: sig.c,v 1.5 1999/07/02 15:21:27 simonb Exp $	*/
+/*	$NetBSD: sig.c,v 1.6 2000/09/04 22:06:32 lukem Exp $	*/
 
 /*-
  * Copyright (c) 1992, 1993
@@ -47,13 +47,13 @@
 private EditLine *sel = NULL;
 
 private int sighdl[] = {
-#define _DO(a)	(a),
-    ALLSIGS
-#undef _DO
-    -1
+#define	_DO(a)	(a),
+	ALLSIGS
+#undef	_DO
+	- 1
 };
 
-private void sig_handler	__P((int));
+private void sig_handler(int);
 
 /* sig_handler():
  *	This is the handler called for all signals
@@ -61,40 +61,39 @@ private void sig_handler	__P((int));
  *	state in a private variable
  */
 private void
-sig_handler(signo)
-    int signo;
+sig_handler(int signo)
 {
-    int i;
-    sigset_t nset, oset;
+	int i;
+	sigset_t nset, oset;
 
-    (void) sigemptyset(&nset);
-    (void) sigaddset(&nset, signo);
-    (void) sigprocmask(SIG_BLOCK, &nset, &oset);
+	(void) sigemptyset(&nset);
+	(void) sigaddset(&nset, signo);
+	(void) sigprocmask(SIG_BLOCK, &nset, &oset);
 
-    switch (signo) {
-    case SIGCONT:
-	tty_rawmode(sel);
-	if (ed_redisplay(sel, 0) == CC_REFRESH)
-	    re_refresh(sel);
-	term__flush();
-	break;
+	switch (signo) {
+	case SIGCONT:
+		tty_rawmode(sel);
+		if (ed_redisplay(sel, 0) == CC_REFRESH)
+			re_refresh(sel);
+		term__flush();
+		break;
 
-    case SIGWINCH:
-	el_resize(sel);
-	break;
+	case SIGWINCH:
+		el_resize(sel);
+		break;
 
-    default:
-	tty_cookedmode(sel);
-	break;
-    }
+	default:
+		tty_cookedmode(sel);
+		break;
+	}
 
-    for (i = 0; sighdl[i] != -1; i++)
-	if (signo == sighdl[i])
-	    break;
+	for (i = 0; sighdl[i] != -1; i++)
+		if (signo == sighdl[i])
+			break;
 
-    (void) xsignal_restart(signo, sel->el_signal[i], 1);
-    (void) sigprocmask(SIG_SETMASK, &oset, NULL);
-    (void) kill(0, signo);
+	(void) xsignal_restart(signo, sel->el_signal[i], 1);
+	(void) sigprocmask(SIG_SETMASK, &oset, NULL);
+	(void) kill(0, signo);
 }
 
 
@@ -102,27 +101,26 @@ sig_handler(signo)
  *	Initialize all signal stuff
  */
 protected int
-sig_init(el)
-    EditLine *el;
+sig_init(EditLine *el)
 {
-    int i;
-    sigset_t nset, oset;
+	int i;
+	sigset_t nset, oset;
 
-    (void) sigemptyset(&nset);
-#define _DO(a) (void) sigaddset(&nset, SIGWINCH);
-    ALLSIGS
-#undef _DO
-    (void) sigprocmask(SIG_BLOCK, &nset, &oset);
+	(void) sigemptyset(&nset);
+#define	_DO(a) (void) sigaddset(&nset, SIGWINCH);
+	ALLSIGS
+#undef	_DO
+	    (void) sigprocmask(SIG_BLOCK, &nset, &oset);
 
-#define SIGSIZE (sizeof(sighdl) / sizeof(sighdl[0]) * sizeof(sigfunc))
+#define	SIGSIZE (sizeof(sighdl) / sizeof(sighdl[0]) * sizeof(sigfunc))
 
-    el->el_signal = (sigfunc *) el_malloc(SIGSIZE);
-    for (i = 0; sighdl[i] != -1; i++)
-	el->el_signal[i] = SIG_ERR;
+	el->el_signal = (sigfunc *) el_malloc(SIGSIZE);
+	for (i = 0; sighdl[i] != -1; i++)
+		el->el_signal[i] = SIG_ERR;
 
-    (void) sigprocmask(SIG_SETMASK, &oset, NULL);
+	(void) sigprocmask(SIG_SETMASK, &oset, NULL);
 
-    return 0;
+	return (0);
 }
 
 
@@ -130,11 +128,11 @@ sig_init(el)
  *	Clear all signal stuff
  */
 protected void
-sig_end(el)
-    EditLine *el;
+sig_end(EditLine *el)
 {
-    el_free((ptr_t) el->el_signal);
-    el->el_signal = NULL;
+
+	el_free((ptr_t) el->el_signal);
+	el->el_signal = NULL;
 }
 
 
@@ -142,26 +140,25 @@ sig_end(el)
  *	set all the signal handlers
  */
 protected void
-sig_set(el)
-    EditLine *el;
+sig_set(EditLine *el)
 {
-    int i;
-    sigset_t nset, oset;
+	int i;
+	sigset_t nset, oset;
 
-    (void) sigemptyset(&nset);
-#define _DO(a) (void) sigaddset(&nset, SIGWINCH);
-    ALLSIGS
-#undef _DO
-    (void) sigprocmask(SIG_BLOCK, &nset, &oset);
+	(void) sigemptyset(&nset);
+#define	_DO(a) (void) sigaddset(&nset, SIGWINCH);
+	ALLSIGS
+#undef	_DO
+	    (void) sigprocmask(SIG_BLOCK, &nset, &oset);
 
-    for (i = 0; sighdl[i] != -1; i++) {
-	sigfunc s;
-	/* This could happen if we get interrupted */
-	if ((s = xsignal_restart(sighdl[i], sig_handler, 1)) != sig_handler)
-	    el->el_signal[i] = s;
-    }
-    sel = el;
-    (void) sigprocmask(SIG_SETMASK, &oset, NULL);
+	for (i = 0; sighdl[i] != -1; i++) {
+		sigfunc s;
+		/* This could happen if we get interrupted */
+		if ((s = xsignal_restart(sighdl[i], sig_handler, 1)) != sig_handler)
+			el->el_signal[i] = s;
+	}
+	sel = el;
+	(void) sigprocmask(SIG_SETMASK, &oset, NULL);
 }
 
 
@@ -169,22 +166,22 @@ sig_set(el)
  *	clear all the signal handlers
  */
 protected void
-sig_clr(el)
-    EditLine *el;
+sig_clr(EditLine *el)
 {
-    int i;
-    sigset_t nset, oset;
+	int i;
+	sigset_t nset, oset;
 
-    (void) sigemptyset(&nset);
-#define _DO(a) (void) sigaddset(&nset, SIGWINCH);
-    ALLSIGS
-#undef _DO
-    (void) sigprocmask(SIG_BLOCK, &nset, &oset);
+	(void) sigemptyset(&nset);
+#define	_DO(a) (void) sigaddset(&nset, SIGWINCH);
+	ALLSIGS
+#undef	_DO
+	    (void) sigprocmask(SIG_BLOCK, &nset, &oset);
 
-    for (i = 0; sighdl[i] != -1; i++)
-	if (el->el_signal[i] != SIG_ERR)
-	    (void) xsignal_restart(sighdl[i], el->el_signal[i], 1);
+	for (i = 0; sighdl[i] != -1; i++)
+		if (el->el_signal[i] != SIG_ERR)
+			(void) xsignal_restart(sighdl[i], el->el_signal[i], 1);
 
-    sel = NULL;	/* we are going to die if the handler is called */
-    (void) sigprocmask(SIG_SETMASK, &oset, NULL);
+	sel = NULL;		/* we are going to die if the handler is
+				 * called */
+	(void) sigprocmask(SIG_SETMASK, &oset, NULL);
 }
