@@ -1,4 +1,4 @@
-/*	$NetBSD: sig.h,v 1.2 1997/01/11 06:48:11 lukem Exp $	*/
+/*	$NetBSD: sig.h,v 1.3 2000/09/04 22:06:32 lukem Exp $	*/
 
 /*-
  * Copyright (c) 1992, 1993
@@ -42,7 +42,7 @@
  * el.sig.h: Signal handling functions
  */
 #ifndef _h_el_sig
-#define _h_el_sig
+#define	_h_el_sig
 
 #include "histedit.h"
 
@@ -50,28 +50,26 @@
  * Define here all the signals we are going to handle
  * The _DO macro is used to iterate in the source code
  */
-#define ALLSIGS 	\
-    _DO(SIGINT)		\
-    _DO(SIGTSTP)	\
-    _DO(SIGSTOP)	\
-    _DO(SIGQUIT)	\
-    _DO(SIGHUP)		\
-    _DO(SIGTERM)	\
-    _DO(SIGCONT)	\
-    _DO(SIGWINCH)
+#define	ALLSIGS		\
+	_DO(SIGINT)	\
+	_DO(SIGTSTP)	\
+	_DO(SIGSTOP)	\
+	_DO(SIGQUIT)	\
+	_DO(SIGHUP)	\
+	_DO(SIGTERM)	\
+	_DO(SIGCONT)	\
+	_DO(SIGWINCH)
 
-
-typedef void (*sigfunc) __P((int));
+typedef void	(*sigfunc)(int);
 
 			/* provided by ../src/util.c */
-sigfunc	xsignal_restart __P((int, sigfunc, int));
+sigfunc	xsignal_restart(int, sigfunc, int);
 
+typedef sigfunc	*el_signal_t;
 
-typedef sigfunc *el_signal_t;
-
-protected void	sig_end		__P((EditLine*));
-protected int	sig_init	__P((EditLine*));
-protected void	sig_set		__P((EditLine*));
-protected void	sig_clr		__P((EditLine*));
+protected void	sig_end(EditLine*);
+protected int	sig_init(EditLine*);
+protected void	sig_set(EditLine*);
+protected void	sig_clr(EditLine*);
 
 #endif /* _h_el_sig */
