@@ -1,10 +1,9 @@
-/*	$Id: strerror.c,v 1.1 1999/09/29 13:51:34 lukem Exp $	*/
+/*	$Id: strerror.c,v 1.2 2000/06/04 14:48:18 lukem Exp $	*/
 
 #include "lukemftp.h"
 
 char *
-strerror(n)
-	int n;
+strerror(int n)
 {
 	static char msg[] = "Unknown error (1234567890)";
 
