@@ -1,4 +1,4 @@
-/*	$Id: strvis.c,v 1.2 2000/06/04 14:48:19 lukem Exp $	*/
+/*	$Id: strvis.c,v 1.3 2000/09/14 13:49:55 lukem Exp $	*/
 /*	$NetBSD: vis.c,v 1.13 1999/09/20 04:39:07 lukem Exp $	*/
 
 /*-
@@ -71,11 +71,7 @@ vis(char *dst, int c, int flag, int nextc)
 			*dst++ = '\\';
 			*dst++ = 'b';
 			goto done;
-#if __STDC__
 		case '\a':
-#else
-		case '\007':
-#endif
 			*dst++ = '\\';
 			*dst++ = 'a';
 			goto done;
