@@ -1,4 +1,4 @@
-/*	$NetBSD: cmds.c,v 1.87 2000/06/15 13:08:23 lukem Exp $	*/
+/*	$NetBSD: cmds.c,v 1.90 2000/08/01 22:47:25 lukem Exp $	*/
 
 /*-
  * Copyright (c) 1996-2000 The NetBSD Foundation, Inc.
@@ -188,7 +188,7 @@ settype(int argc, char *argv[])
 	struct types *p;
 	int comret;
 
-	if ((argc == 0 && argv != NULL) || argc > 2) {
+	if (argc == 0 || argc > 2) {
 		char *sep;
 
 		fprintf(ttyout, "usage: %s [", argv[0]);
@@ -271,7 +271,7 @@ void
 setbinary(int argc, char *argv[])
 {
 
-	if (argc == 0 && argv != NULL) {
+	if (argc == 0) {
 		fprintf(ttyout, "usage: %s\n", argv[0]);
 		code = -1;
 		return;
@@ -288,7 +288,7 @@ void
 setascii(int argc, char *argv[])
 {
 
-	if (argc == 0 && argv != NULL) {
+	if (argc == 0) {
 		fprintf(ttyout, "usage: %s\n", argv[0]);
 		code = -1;
 		return;
@@ -305,7 +305,7 @@ void
 settenex(int argc, char *argv[])
 {
 
-	if (argc == 0 && argv != NULL) {
+	if (argc == 0) {
 		fprintf(ttyout, "usage: %s\n", argv[0]);
 		code = -1;
 		return;
@@ -322,7 +322,7 @@ void
 setftmode(int argc, char *argv[])
 {
 
-	if ((argc == 0 && argv != NULL) || argc != 2) {
+	if (argc != 2) {
 		fprintf(ttyout, "usage: %s mode-name\n", argv[0]);
 		code = -1;
 		return;
@@ -339,7 +339,7 @@ void
 setform(int argc, char *argv[])
 {
 
-	if ((argc == 0 && argv != NULL) || argc != 2) {
+	if (argc != 2) {
 		fprintf(ttyout, "usage: %s format\n", argv[0]);
 		code = -1;
 		return;
@@ -356,7 +356,7 @@ void
 setstruct(int argc, char *argv[])
 {
 
-	if ((argc == 0 && argv != NULL) || argc != 2) {
+	if (argc != 2) {
 		fprintf(ttyout, "usage: %s struct-mode\n", argv[0]);
 		code = -1;
 		return;
@@ -380,11 +380,10 @@ put(int argc, char *argv[])
 		argv[2] = argv[1];
 		loc++;
 	}
-	if ((argc == 0 && argv != NULL) ||
-	    (argc == 1 && !another(&argc, &argv, "local-file")))
+	if (argc == 0 || (argc == 1 && !another(&argc, &argv, "local-file")))
 		goto usage;
 	if ((argc < 3 && !another(&argc, &argv, "remote-file")) || argc > 3) {
-usage:
+ usage:
 		fprintf(ttyout, "usage: %s local-file [remote-file]\n",
 		    argv[0]);
 		code = -1;
@@ -418,8 +417,7 @@ mput(int argc, char *argv[])
 	int ointer;
 	char *tp;
 
-	if ((argc == 0 && argv != NULL) ||
-	    (argc == 1 && !another(&argc, &argv, "local-files"))) {
+	if (argc == 0 || (argc == 1 && !another(&argc, &argv, "local-files"))) {
 		fprintf(ttyout, "usage: %s local-files\n", argv[0]);
 		code = -1;
 		return;
@@ -508,7 +506,7 @@ mput(int argc, char *argv[])
 		}
 		globfree(&gl);
 	}
-cleanupmput:
+ cleanupmput:
 	(void)xsignal(SIGINT, oldintr);
 	mflag = 0;
 }
@@ -542,11 +540,10 @@ getit(int argc, char *argv[], int restartit, const char *mode)
 		argv[2] = argv[1];
 		loc++;
 	}
-	if ((argc == 0 && argv != NULL) ||
-	    (argc == 1 && !another(&argc, &argv, "remote-file")))
+	if (argc == 0 || (argc == 1 && !another(&argc, &argv, "remote-file")))
 		goto usage;
 	if ((argc < 3 && !another(&argc, &argv, "local-file")) || argc > 3) {
-usage:
+ usage:
 		fprintf(ttyout, "usage: %s remote-file [local-file]\n",
 		    argv[0]);
 		code = -1;
@@ -568,6 +565,11 @@ usage:
 		struct stat stbuf;
 		int ret;
 
+		if (! features[FEAT_REST_STREAM]) {
+			fprintf(ttyout,
+			    "Restart is not supported by the remote server.\n");
+			return (0);
+		}
 		ret = stat(locfile, &stbuf);
 		if (restartit == 1) {
 			if (ret < 0) {
@@ -593,7 +595,7 @@ usage:
 	recvrequest("RETR", locfile, remfile, mode,
 	    remfile != argv[1] || locfile != argv[2], loc);
 	restart_point = 0;
-freegetit:
+ freegetit:
 	(void)free(olocfile);
 	return (rval);
 }
@@ -640,7 +642,7 @@ mget(int argc, char *argv[])
 	int ch, ointer;
 	char *cp, *tp, *tp2, tmpbuf[MAXPATHLEN];
 
-	if ((argc == 0 && argv != NULL) ||
+	if (argc == 0 ||
 	    (argc == 1 && !another(&argc, &argv, "remote-files"))) {
 		fprintf(ttyout, "usage: %s remote-files\n", argv[0]);
 		code = -1;
@@ -738,7 +740,7 @@ status(int argc, char *argv[])
 {
 	int i;
 
-	if (argc == 0 && argv != NULL) {
+	if (argc == 0) {
 		fprintf(ttyout, "usage: %s\n", argv[0]);
 		code = -1;
 		return;
@@ -857,7 +859,7 @@ setedit(int argc, char *argv[])
 {
 
 #ifdef NO_EDITCOMPLETE
-	if (argc == 0 && argv != NULL) {
+	if (argc == 0) {
 		fprintf(ttyout, "usage: %s\n", argv[0]);
 		code = -1;
 		return;
@@ -977,7 +979,7 @@ setgate(int argc, char *argv[])
 {
 	static char gsbuf[MAXHOSTNAMELEN];
 
-	if ((argc == 0 && argv != NULL) || argc > 3) {
+	if (argc == 0 || argc > 3) {
 		fprintf(ttyout,
 		    "usage: %s [ on | off | gateserver [port] ]\n", argv[0]);
 		code = -1;
@@ -1038,7 +1040,7 @@ setpreserve(int argc, char *argv[])
 void
 setdebug(int argc, char *argv[])
 {
-	if ((argc == 0 && argv != NULL) || argc > 2) {
+	if (argc == 0 || argc > 2) {
 		fprintf(ttyout, "usage: %s [ on | off | debuglevel ]\n",
 		    argv[0]);
 		code = -1;
@@ -1078,8 +1080,8 @@ cd(int argc, char *argv[])
 {
 	int r;
 
-	if ((argc == 1 && !another(&argc, &argv, "remote-directory")) ||
-	    (argc == 0 && argv != NULL) || argc > 2) {
+	if (argc == 0 || argc > 2 ||
+	    (argc == 1 && !another(&argc, &argv, "remote-directory"))) {
 		fprintf(ttyout, "usage: %s remote-directory\n", argv[0]);
 		code = -1;
 		return;
@@ -1137,8 +1139,8 @@ delete(int argc, char *argv[])
 {
 
 
-	if ((argc == 1 && !another(&argc, &argv, "remote-file")) ||
-	    (argc == 0 && argv != NULL) || argc > 2) {
+	if (argc == 0 || argc > 2 ||
+	    (argc == 1 && !another(&argc, &argv, "remote-file"))) {
 		fprintf(ttyout, "usage: %s remote-file\n", argv[0]);
 		code = -1;
 		return;
@@ -1156,8 +1158,8 @@ mdelete(int argc, char *argv[])
 	int ointer;
 	char *cp;
 
-	if ((argc == 1 && !another(&argc, &argv, "remote-files")) ||
-	    (argc == 0 && argv != NULL)) {
+	if (argc == 0 ||
+	    (argc == 1 && !another(&argc, &argv, "remote-files"))) {
 		fprintf(ttyout, "usage: %s [remote-files]\n", argv[0]);
 		code = -1;
 		return;
@@ -1195,11 +1197,10 @@ void
 renamefile(int argc, char *argv[])
 {
 
-	if ((argc == 0 && argv != NULL) ||
-	    (argc == 1 && !another(&argc, &argv, "from-name")))
+	if (argc == 0 || (argc == 1 && !another(&argc, &argv, "from-name")))
 		goto usage;
 	if ((argc < 3 && !another(&argc, &argv, "to-name")) || argc > 3) {
-usage:
+ usage:
 		fprintf(ttyout, "usage: %s from-name to-name\n", argv[0]);
 		code = -1;
 		return;
@@ -1210,42 +1211,61 @@ usage:
 
 /*
  * Get a directory listing of remote files.
+ * Supports being invoked as:
+ *	cmd		runs
+ *	---		----
+ *	dir, ls		LIST
+ *	mlsd		MLSD
+ *	nlist		NLST
+ *	pdir, pls	LIST |$PAGER
+ *	mmlsd		MLSD |$PAGER
  */
 void
 ls(int argc, char *argv[])
 {
 	const char *cmd;
 	char *remdir, *locfile;
-	int freelocfile, pagecmd;
+	int freelocfile, pagecmd, mlsdcmd;
 
 	remdir = NULL;
 	locfile = "-";
-	freelocfile = pagecmd = 0;
+	freelocfile = pagecmd = mlsdcmd = 0;
 			/*
-			 * assume all `pager' versions of the commands
-			 * are the only ones that start with `p'
+			 * the only commands that start with `p' are
+			 * the `pager' versions.
 			 */
 	if (argv[0][0] == 'p')
 		pagecmd = 1;
-	if (argc == 0 && argv != NULL)
+	if (strcmp(argv[0] + pagecmd , "mlsd") == 0) {
+		if (! features[FEAT_MLST]) {
+			fprintf(ttyout,
+			   "MLSD is not supported by the remote server.\n");
+			return;
+		}
+		mlsdcmd = 1;
+	}
+	if (argc == 0)
 		goto usage;
 
-	cmd = "LIST";
-	if (strcmp(argv[0], "nlist") == 0)
+	if (mlsdcmd)
+		cmd = "MLSD";
+	else if (strcmp(argv[0] + pagecmd, "nlist") == 0)
 		cmd = "NLST";
+	else
+		cmd = "LIST";
 
 	if (argc > 1)
 		remdir = argv[1];
 	if (argc > 2)
 		locfile = argv[2];
-	if (argc > 3 || (pagecmd && argc > 2)) {
+	if (argc > 3 || ((pagecmd | mlsdcmd) && argc > 2)) {
  usage:
-		if (pagecmd)
+		if (pagecmd || mlsdcmd)
 			fprintf(ttyout,
-			    "usage: %s [remote-directory]\n", argv[0]);
+			    "usage: %s [remote-path]\n", argv[0]);
 		else
 			fprintf(ttyout,
-			    "usage: %s [remote-directory [local-file]]\n",
+			    "usage: %s [remote-path [local-file]]\n",
 			    argv[0]);
 		code = -1;
 		goto freels;
@@ -1272,7 +1292,7 @@ ls(int argc, char *argv[])
 		freelocfile = 1;
 	}
 	recvrequest(cmd, locfile, remdir, "w", 0, 0);
-freels:
+ freels:
 	if (freelocfile && locfile)
 		(void)free(locfile);
 }
@@ -1288,12 +1308,12 @@ mls(int argc, char *argv[])
 	int dolist;
 	char mode[1], *dest, *odest;
 
-	if (argc == 0 && argv != NULL)
+	if (argc == 0)
 		goto usage;
 	if (argc < 2 && !another(&argc, &argv, "remote-files"))
 		goto usage;
 	if (argc < 3 && !another(&argc, &argv, "local-file")) {
-usage:
+ usage:
 		fprintf(ttyout, "usage: %s remote-files local-file\n", argv[0]);
 		code = -1;
 		return;
@@ -1343,7 +1363,7 @@ shell(int argc, char *argv[])
 	char shellnam[MAXPATHLEN], *shell, *namep;
 	int wait_status;
 
-	if (argc == 0 && argv != NULL) {
+	if (argc == 0) {
 		fprintf(ttyout, "usage: %s [command [args]]\n", argv[0]);
 		code = -1;
 		return;
@@ -1396,7 +1416,7 @@ user(int argc, char *argv[])
 	char acct[80];
 	int n, aflag = 0;
 
-	if (argc == 0 && argv != NULL)
+	if (argc == 0)
 		goto usage;
 	if (argc < 2)
 		(void)another(&argc, &argv, "username");
@@ -1440,7 +1460,7 @@ user(int argc, char *argv[])
 		(void)command("ACCT %s", argv[3]);
 	}
 	connected = -1;
-	remotesyst();
+	getremoteinfo();
 }
 
 /*
@@ -1452,15 +1472,12 @@ pwd(int argc, char *argv[])
 {
 	int oldverbose = verbose;
 
-	if (argc == 0 && argv != NULL) {
+	if (argc == 0) {
 		fprintf(ttyout, "usage: %s\n", argv[0]);
 		code = -1;
 		return;
 	}
-	/*
-	 * If we aren't verbose, this doesn't do anything!
-	 */
-	verbose = 1;
+	verbose = 1;	/* If we aren't verbose, this doesn't do anything! */
 	if (command("PWD") == ERROR && code == 500) {
 		fputs("PWD command not recognized, trying XPWD.\n", ttyout);
 		(void)command("XPWD");
@@ -1476,7 +1493,7 @@ lpwd(int argc, char *argv[])
 {
 	char buf[MAXPATHLEN];
 
-	if (argc == 0 && argv != NULL) {
+	if (argc == 0) {
 		fprintf(ttyout, "usage: %s\n", argv[0]);
 		code = -1;
 		return;
@@ -1497,8 +1514,8 @@ void
 makedir(int argc, char *argv[])
 {
 
-	if ((argc == 1 && !another(&argc, &argv, "directory-name")) ||
-	    (argc == 0 && argv != NULL) || argc > 2) {
+	if (argc == 0 || argc > 2 ||
+	    (argc == 1 && !another(&argc, &argv, "directory-name"))) {
 		fprintf(ttyout, "usage: %s directory-name\n", argv[0]);
 		code = -1;
 		return;
@@ -1518,8 +1535,8 @@ void
 removedir(int argc, char *argv[])
 {
 
-	if ((argc == 1 && !another(&argc, &argv, "directory-name")) ||
-	    (argc == 0 && argv != NULL) || argc > 2) {
+	if (argc == 0 || argc > 2 ||
+	    (argc == 1 && !another(&argc, &argv, "directory-name"))) {
 		fprintf(ttyout, "usage: %s directory-name\n", argv[0]);
 		code = -1;
 		return;
@@ -1539,8 +1556,8 @@ void
 quote(int argc, char *argv[])
 {
 
-	if ((argc == 1 && !another(&argc, &argv, "command line to send")) ||
-	    (argc == 0 && argv != NULL)) {
+	if (argc == 0 ||
+	    (argc == 1 && !another(&argc, &argv, "command line to send"))) {
 		fprintf(ttyout, "usage: %s line-to-send\n", argv[0]);
 		code = -1;
 		return;
@@ -1557,8 +1574,8 @@ void
 site(int argc, char *argv[])
 {
 
-	if ((argc == 1 && !another(&argc, &argv, "arguments to SITE command"))
-	    || (argc == 0 && argv != NULL)) {
+	if (argc == 0 ||
+	    (argc == 1 && !another(&argc, &argv, "arguments to SITE command"))){
 		fprintf(ttyout, "usage: %s line-to-send\n", argv[0]);
 		code = -1;
 		return;
@@ -1592,11 +1609,10 @@ void
 do_chmod(int argc, char *argv[])
 {
 
-	if ((argc == 0 && argv != NULL) ||
-	    (argc == 1 && !another(&argc, &argv, "mode")))
+	if (argc == 0 || (argc == 1 && !another(&argc, &argv, "mode")))
 		goto usage;
 	if ((argc < 3 && !another(&argc, &argv, "remote-file")) || argc > 3) {
-usage:
+ usage:
 		fprintf(ttyout, "usage: %s mode remote-file\n", argv[0]);
 		code = -1;
 		return;
@@ -1609,7 +1625,7 @@ do_umask(int argc, char *argv[])
 {
 	int oldverbose = verbose;
 
-	if (argc == 0 && argv != NULL) {
+	if (argc == 0) {
 		fprintf(ttyout, "usage: %s [umask]\n", argv[0]);
 		code = -1;
 		return;
@@ -1642,7 +1658,7 @@ rmthelp(int argc, char *argv[])
 {
 	int oldverbose = verbose;
 
-	if (argc == 0 && argv != NULL) {
+	if (argc == 0) {
 		fprintf(ttyout, "usage: %s\n", argv[0]);
 		code = -1;
 		return;
@@ -1654,12 +1670,14 @@ rmthelp(int argc, char *argv[])
 
 /*
  * Terminate session and exit.
+ * May be called with 0, NULL.
  */
 /*VARARGS*/
 void
 quit(int argc, char *argv[])
 {
 
+			/* this may be called with argc == 0, argv == NULL */
 	if (argc == 0 && argv != NULL) {
 		fprintf(ttyout, "usage: %s\n", argv[0]);
 		code = -1;
@@ -1675,11 +1693,13 @@ quit(int argc, char *argv[])
 
 /*
  * Terminate session, but don't exit.
+ * May be called with 0, NULL.
  */
 void
 disconnect(int argc, char *argv[])
 {
 
+			/* this may be called with argc == 0, argv == NULL */
 	if (argc == 0 && argv != NULL) {
 		fprintf(ttyout, "usage: %s\n", argv[0]);
 		code = -1;
@@ -1696,7 +1716,7 @@ account(int argc, char *argv[])
 {
 	char *ap;
 
-	if ((argc == 0 && argv != NULL) || argc > 2) {
+	if (argc == 0 || argc > 2) {
 		fprintf(ttyout, "usage: %s [password]\n", argv[0]);
 		code = -1;
 		return;
@@ -1735,8 +1755,7 @@ doproxy(int argc, char *argv[])
 	int cmdpos;
 	sigfunc oldintr;
 
-	if ((argc == 0 && argv != NULL) ||
-	    (argc == 1 && !another(&argc, &argv, "command"))) {
+	if (argc == 0 || (argc == 1 && !another(&argc, &argv, "command"))) {
 		fprintf(ttyout, "usage: %s command\n", argv[0]);
 		code = -1;
 		return;
@@ -1828,7 +1847,7 @@ void
 setntrans(int argc, char *argv[])
 {
 
-	if ((argc == 0 && argv != NULL) || argc > 3) {
+	if (argc == 0 || argc > 3) {
 		fprintf(ttyout, "usage: %s [inchars [outchars]]\n", argv[0]);
 		code = -1;
 		return;
@@ -1888,7 +1907,7 @@ setnmap(int argc, char *argv[])
 		code = mapflag;
 		return;
 	}
-	if ((argc == 0 && argv != NULL) ||
+	if (argc == 0 ||
 	    (argc < 3 && !another(&argc, &argv, "mapout")) || argc > 3) {
 		fprintf(ttyout, "usage: %s [mapin mapout]\n", argv[0]);
 		code = -1;
@@ -2142,7 +2161,7 @@ parserate(int argc, char *argv[], int cmdlineopt)
 	sigfunc oldusr1, oldusr2;
 
 	if (argc > 4 || (argc < (cmdlineopt ? 3 : 2))) {
-usage:
+ usage:
 		if (cmdlineopt)
 			fprintf(ttyout,
 	"usage: %s (all|get|put),maximum-bytes[,increment-bytes]]\n",
@@ -2219,7 +2238,7 @@ cdup(int argc, char *argv[])
 {
 	int r;
 
-	if (argc == 0 && argv != NULL) {
+	if (argc == 0) {
 		fprintf(ttyout, "usage: %s\n", argv[0]);
 		code = -1;
 		return;
@@ -2244,20 +2263,21 @@ void
 restart(int argc, char *argv[])
 {
 
-	if ((argc == 0 && argv != NULL) || argc > 2) {
-		fprintf(ttyout, "usage: %s [restart_point]\n", argv[0]);
+	if (argc == 0 || argc > 2) {
+		fprintf(ttyout, "usage: %s [restart-point]\n", argv[0]);
 		code = -1;
 		return;
 	}
+	if (! features[FEAT_REST_STREAM]) {
+		fprintf(ttyout,
+		    "Restart is not supported by the remote server.\n");
+		return;
+	}
 	if (argc == 2) {
 		off_t rp;
 		char *ep;
 
-#ifndef NO_QUAD
-		rp = strtoll(argv[1], &ep, 10);
-#else
-		rp = strtol(argv[1], &ep, 10);
-#endif
+		rp = STRTOLL(argv[1], &ep, 10);
 		if (rp < 0 || *ep != '\0')
 			fprintf(ttyout, "restart: Invalid offset `%s'\n",
 			    argv[1]);
@@ -2268,13 +2288,8 @@ restart(int argc, char *argv[])
 		fputs("No restart point defined.\n", ttyout);
 	else
 		fprintf(ttyout,
-#ifndef NO_QUAD
-		    "Restarting at %lld for next get, put or append\n",
-		    (long long)restart_point);
-#else
-		    "Restarting at %ld for next get, put or append\n",
-		    (long)restart_point);
-#endif
+		    "Restarting at " LLF " for next get, put or append\n",
+		    (LLT)restart_point);
 }
 
 /*
@@ -2283,13 +2298,16 @@ restart(int argc, char *argv[])
 void
 syst(int argc, char *argv[])
 {
+	int oldverbose = verbose;
 
-	if (argc == 0 && argv != NULL) {
+	if (argc == 0) {
 		fprintf(ttyout, "usage: %s\n", argv[0]);
 		code = -1;
 		return;
 	}
+	verbose = 1;	/* If we aren't verbose, this doesn't do anything! */
 	(void)command("SYST");
+	verbose = oldverbose;
 }
 
 void
@@ -2298,7 +2316,7 @@ macdef(int argc, char *argv[])
 	char *tmp;
 	int c;
 
-	if (argc == 0 && argv != NULL)
+	if (argc == 0)
 		goto usage;
 	if (macnum == 16) {
 		fputs("Limit of 16 macros have already been defined.\n",
@@ -2364,8 +2382,8 @@ sizecmd(int argc, char *argv[])
 {
 	off_t size;
 
-	if ((argc == 1 && !another(&argc, &argv, "remote-file")) ||
-	    (argc == 0 && argv != NULL) || argc > 2) {
+	if (argc == 0 || argc > 2 ||
+	    (argc == 1 && !another(&argc, &argv, "remote-file"))) {
 		fprintf(ttyout, "usage: %s remote-file\n", argv[0]);
 		code = -1;
 		return;
@@ -2373,11 +2391,7 @@ sizecmd(int argc, char *argv[])
 	size = remotesize(argv[1], 1);
 	if (size != -1)
 		fprintf(ttyout,
-#ifndef NO_QUAD
-		    "%s\t%lld\n", argv[1], (long long)size);
-#else
-		    "%s\t%ld\n", argv[1], (long)size);
-#endif
+		    "%s\t" LLF "\n", argv[1], (LLT)size);
 	code = (size > 0);
 }
 
@@ -2389,8 +2403,8 @@ modtime(int argc, char *argv[])
 {
 	time_t mtime;
 
-	if ((argc == 1 && !another(&argc, &argv, "remote-file")) ||
-	    (argc == 0 && argv != NULL) || argc > 2) {
+	if (argc == 0 || argc > 2 ||
+	    (argc == 1 && !another(&argc, &argv, "remote-file"))) {
 		fprintf(ttyout, "usage: %s remote-file\n", argv[0]);
 		code = -1;
 		return;
@@ -2408,7 +2422,7 @@ void
 rmtstatus(int argc, char *argv[])
 {
 
-	if (argc == 0 && argv != NULL) {
+	if (argc == 0) {
 		fprintf(ttyout, "usage: %s [remote-file]\n", argv[0]);
 		code = -1;
 		return;
@@ -2438,8 +2452,8 @@ lpage(int argc, char *argv[])
 	int len;
 	char *p, *pager, *locfile;
 
-	if ((argc == 0 && argv != NULL) ||
-	    (argc == 1 && !another(&argc, &argv, "local-file")) || argc > 2) {
+	if (argc == 0 || argc > 2 ||
+	    (argc == 1 && !another(&argc, &argv, "local-file"))) {
 		fprintf(ttyout, "usage: %s local-file\n", argv[0]);
 		code = -1;
 		return;
@@ -2471,8 +2485,8 @@ page(int argc, char *argv[])
 	int ohash, orestart_point, overbose, len;
 	char *p, *pager;
 
-	if ((argc == 0 && argv != NULL) ||
-	    (argc == 1 && !another(&argc, &argv, "remote-file")) || argc > 2) {
+	if (argc == 0 || argc > 2 ||
+	    (argc == 1 && !another(&argc, &argv, "remote-file"))) {
 		fprintf(ttyout, "usage: %s remote-file\n", argv[0]);
 		code = -1;
 		return;
@@ -2545,7 +2559,7 @@ setoption(int argc, char *argv[])
 	struct option *o;
 
 	code = -1;
-	if ((argc == 0 && argv != NULL) || (argc != 1 && argc != 3)) {
+	if (argc == 0 || (argc != 1 && argc != 3)) {
 		fprintf(ttyout, "usage: %s [option value]\n", argv[0]);
 		return;
 	}
@@ -2580,7 +2594,7 @@ unsetoption(int argc, char *argv[])
 	struct option *o;
 
 	code = -1;
-	if ((argc == 0 && argv != NULL) || argc !=  2) {
+	if (argc == 0 || argc != 2) {
 		fprintf(ttyout, "usage: %s option\n", argv[0]);
 		return;
 	}
@@ -2594,3 +2608,66 @@ unsetoption(int argc, char *argv[])
 	fprintf(ttyout, "Unsetting `%s'.\n", o->name);
 	code = 0;
 }
+
+/*
+ * Display features supported by the remote host.
+ */
+void
+feat(int argc, char *argv[])
+{
+	int oldverbose = verbose;
+
+	if (argc == 0) {
+		fprintf(ttyout, "usage: %s\n", argv[0]);
+		code = -1;
+		return;
+	}
+	if (! features[FEAT_FEAT]) {
+		fprintf(ttyout,
+		    "FEAT is not supported by the remote server.\n");
+		return;
+	}
+	verbose = 1;	/* If we aren't verbose, this doesn't do anything! */
+	(void)command("FEAT");
+	verbose = oldverbose;
+}
+
+void
+mlst(int argc, char *argv[])
+{
+	int oldverbose = verbose;
+
+	if (argc < 1 || argc > 2) {
+		fprintf(ttyout, "usage: %s [remote-path]\n", argv[0]);
+		code = -1;
+		return;
+	}
+	if (! features[FEAT_MLST]) {
+		fprintf(ttyout,
+		    "MLST is not supported by the remote server.\n");
+		return;
+	}
+	verbose = 1;	/* If we aren't verbose, this doesn't do anything! */
+	(void)command(argc == 1 ? "MLST" : "MLST %s", argv[1]);
+	verbose = oldverbose;
+}
+
+void
+opts(int argc, char *argv[])
+{
+	int oldverbose = verbose;
+
+	if (argc < 2 || argc > 3) {
+		fprintf(ttyout, "usage: %s command [options]\n", argv[0]);
+		code = -1;
+		return;
+	}
+	if (! features[FEAT_FEAT]) {
+		fprintf(ttyout,
+		    "OPTS is not supported by the remote server.\n");
+		return;
+	}
+	verbose = 1;	/* If we aren't verbose, this doesn't do anything! */
+	(void)command(argc == 2 ? "OPTS %s" : "OPTS %s %s", argv[1], argv[2]);
+	verbose = oldverbose;
+}
