@@ -1,4 +1,4 @@
-/*	$NetBSD: fetch.c,v 1.116 2000/06/15 13:08:25 lukem Exp $	*/
+/*	$NetBSD: fetch.c,v 1.125 2000/09/28 12:29:23 lukem Exp $	*/
 
 /*-
  * Copyright (c) 1997-2000 The NetBSD Foundation, Inc.
@@ -44,6 +44,7 @@
  */
 
 #include "lukemftp.h"
+
 #include "ftp_var.h"
 #include "version.h"
 
@@ -158,7 +159,7 @@ auth_url(const char *challenge, char **response, const char *guser,
 	memset(clear, 0, clen);
 	rval = 0;
 
-cleanup_auth_url:
+ cleanup_auth_url:
 	FREEPTR(clear);
 	FREEPTR(line);
 	FREEPTR(realm);
@@ -169,7 +170,7 @@ cleanup_auth_url:
  * Encode len bytes starting at clear using base64 encoding into encoded,
  * which should be at least ((len + 2) * 4 / 3 + 1) in size.
  */
-void
+static void
 base64_encode(const char *clear, size_t len, char *encoded)
 {
 	static const char enc[] =
@@ -283,7 +284,7 @@ parse_url(const char *url, const char *desc, url_t *type,
 		*type = FILE_URL_T;
 	} else {
 		warnx("Invalid %s `%s'", desc, url);
-cleanup_parse_url:
+ cleanup_parse_url:
 		FREEPTR(*user);
 		FREEPTR(*pass);
 		FREEPTR(*host);
@@ -359,20 +360,10 @@ cleanup_parse_url:
 	if (cp != NULL) {
 		long	nport;
 
-		nport = strtol(cp, &ep, 10);
-		if (*ep != '\0' && ep == cp) {
-			struct servent	*svp;
-
-			svp = getservbyname(cp, "tcp");
-			if (svp == NULL) {
-				warnx("Unknown port `%s' in %s `%s'",
-				    cp, desc, origurl);
-				goto cleanup_parse_url;
-			} else
-				nport = ntohs(svp->s_port);
-		} else if (nport < 1 || nport > MAX_IN_PORT_T || *ep != '\0') {
-			warnx("Invalid port `%s' in %s `%s'", cp, desc,
-			    origurl);
+		nport = parseport(cp, -1);
+		if (nport == -1) {
+			warnx("Unknown port `%s' in %s `%s'",
+			    cp, desc, origurl);
 			goto cleanup_parse_url;
 		}
 		*portnum = nport;
@@ -408,14 +399,9 @@ sigjmp_buf	httpabort;
 static int
 fetch_url(const char *url, const char *proxyenv, char *proxyauth, char *wwwauth)
 {
-#if defined(NI_NUMERICHOST) && defined(INET6)
 	struct addrinfo		hints, *res, *res0 = NULL;
 	int			error;
 	char			hbuf[NI_MAXHOST];
-#else
-	struct sockaddr_in	sin;
-	struct hostent		*hp = NULL;
-#endif
 	volatile sigfunc	oldintr, oldintp;
 	volatile int		s;
 	struct stat		sb;
@@ -535,13 +521,8 @@ fetch_url(const char *url, const char *proxyenv, char *proxyauth, char *wwwauth)
 		if (verbose) {
 			fprintf(ttyout, "Copying %s", decodedpath);
 			if (restart_point)
-#ifndef NO_QUAD
-				fprintf(ttyout, " (restarting at %lld)",
-				    (long long)restart_point);
-#else
-				fprintf(ttyout, " (restarting at %ld)",
-				    (long)restart_point);
-#endif
+				fprintf(ttyout, " (restarting at " LLF ")",
+				    (LLT)restart_point);
 			fputs("\n", ttyout);
 		}
 	} else {				/* ftp:// or http:// URLs */
@@ -634,74 +615,14 @@ fetch_url(const char *url, const char *proxyenv, char *proxyauth, char *wwwauth)
 			}
 		} /* ! EMPTYSTRING(proxyenv) */
 
-#if !defined(NI_NUMERICHOST) || !defined(INET6)
-		memset(&sin, 0, sizeof(sin));
-		sin.sin_family = AF_INET;
-
-		if (isdigit((unsigned char)host[0])) {
-			if (inet_aton(host, &sin.sin_addr) == 0) {
-				warnx("Invalid IP address `%s'", host);
-				goto cleanup_fetch_url;
-			}
-		} else {
-			hp = gethostbyname(host);
-			if (hp == NULL) {
-				warnx("%s: %s", host, hstrerror(h_errno));
-				goto cleanup_fetch_url;
-			}
-			if (hp->h_addrtype != AF_INET) {
-				warnx("`%s': not an Internet address?", host);
-				goto cleanup_fetch_url;
-			}
-			if (hp->h_length > sizeof(sin.sin_addr))
-				hp->h_length = sizeof(sin.sin_addr);
-			memcpy(&sin.sin_addr, hp->h_addr, hp->h_length);
-		}
-		sin.sin_port = htons(portnum);
-
-		s = socket(AF_INET, SOCK_STREAM, 0);
-		if (s == -1) {
-			warn("Can't create socket");
-			goto cleanup_fetch_url;
-		}
-
-		while (xconnect(s, (struct sockaddr *)&sin,
-		    sizeof(sin)) == -1) {
-			if (errno == EINTR)
-				continue;
-			if (hp && hp->h_addr_list[1]) {
-				int oerrno = errno;
-				char *ia;
-
-				ia = inet_ntoa(sin.sin_addr);
-				errno = oerrno;
-				warn("Connect to address `%s'", ia);
-				hp->h_addr_list++;
-				memcpy(&sin.sin_addr, hp->h_addr_list[0],
-				    (size_t)hp->h_length);
-				if (verbose)
-					fprintf(ttyout, "Trying %s...\n",
-					    inet_ntoa(sin.sin_addr));
-				(void)close(s);
-				s = socket(AF_INET, SOCK_STREAM, 0);
-				if (s < 0) {
-					warn("Can't create socket");
-					goto cleanup_fetch_url;
-				}
-				continue;
-			}
-			warn("Can't connect to `%s'", host);
-			goto cleanup_fetch_url;
-		}
-#else
 		memset(&hints, 0, sizeof(hints));
 		hints.ai_flags = 0;
 		hints.ai_family = AF_UNSPEC;
 		hints.ai_socktype = SOCK_STREAM;
 		hints.ai_protocol = 0;
-		error = getaddrinfo(host, port, &hints, &res0);
+		error = getaddrinfo(host, NULL, &hints, &res0);
 		if (error) {
-			warnx(gai_strerror(error));
+			warnx("%s", gai_strerror(error));
 			goto cleanup_fetch_url;
 		}
 		if (res0->ai_canonname)
@@ -721,8 +642,10 @@ fetch_url(const char *url, const char *proxyenv, char *proxyauth, char *wwwauth)
 			if (verbose && res != res0)
 				fprintf(ttyout, "Trying %s...\n", hbuf);
 
-			s = socket(res->ai_family, res->ai_socktype,
-				res->ai_protocol);
+			((struct sockaddr_in *)res->ai_addr)->sin_port =
+			    htons(portnum);
+			s = socket(res->ai_family, SOCK_STREAM,
+			    res->ai_protocol);
 			if (s < 0) {
 				warn("Can't create socket");
 				continue;
@@ -744,7 +667,6 @@ fetch_url(const char *url, const char *proxyenv, char *proxyauth, char *wwwauth)
 			warn("Can't connect to %s", host);
 			goto cleanup_fetch_url;
 		}
-#endif
 
 		fin = fdopen(s, "r+");
 		/*
@@ -786,17 +708,10 @@ fetch_url(const char *url, const char *proxyenv, char *proxyauth, char *wwwauth)
 			fprintf(fin, "Connection: close\r\n");
 			if (restart_point) {
 				fputs(leading, ttyout);
-#ifndef NO_QUAD
-				fprintf(fin, "Range: bytes=%lld-\r\n",
-				    (long long)restart_point);
-				fprintf(ttyout, "restarting at %lld",
-				    (long long)restart_point);
-#else
-				fprintf(fin, "Range: bytes=%ld-\r\n",
-				    (long)restart_point);
-				fprintf(ttyout, "restarting at %ld",
-				    (long)restart_point);
-#endif
+				fprintf(fin, "Range: bytes=" LLF "-\r\n",
+				    (LLT)restart_point);
+				fprintf(ttyout, "restarting at " LLF,
+				    (LLT)restart_point);
 				leading = ", ";
 				hasleading++;
 			}
@@ -874,67 +789,38 @@ fetch_url(const char *url, const char *proxyenv, char *proxyauth, char *wwwauth)
 			if (strncasecmp(cp, CONTENTLEN,
 					sizeof(CONTENTLEN) - 1) == 0) {
 				cp += sizeof(CONTENTLEN) - 1;
-#ifndef NO_QUAD
-				filesize = strtoll(cp, &ep, 10);
-#else
-				filesize = strtol(cp, &ep, 10);
-#endif
+				filesize = STRTOLL(cp, &ep, 10);
 				if (filesize < 0 || *ep != '\0')
 					goto improper;
 				if (debug)
-#ifndef NO_QUAD
-					fprintf(ttyout, "parsed len as: %lld\n",
-					    (long long)filesize);
-#else
-					fprintf(ttyout, "parsed len as: %ld\n",
-					    (long)filesize);
-#endif
+					fprintf(ttyout,
+					    "parsed len as: " LLF "\n",
+					    (LLT)filesize);
 
 #define CONTENTRANGE "Content-Range: bytes "
 			} else if (strncasecmp(cp, CONTENTRANGE,
 					sizeof(CONTENTRANGE) - 1) == 0) {
 				cp += sizeof(CONTENTRANGE) - 1;
-#ifndef NO_QUAD
-				rangestart = strtoll(cp, &ep, 10);
-#else
-				rangestart = strtol(cp, &ep, 10);
-#endif
+				rangestart = STRTOLL(cp, &ep, 10);
 				if (rangestart < 0 || *ep != '-')
 					goto improper;
 				cp = ep + 1;
-
-#ifndef NO_QUAD
-				rangeend = strtoll(cp, &ep, 10);
-#else
-				rangeend = strtol(cp, &ep, 10);
-#endif
+				rangeend = STRTOLL(cp, &ep, 10);
 				if (rangeend < 0 || *ep != '/' ||
 				    rangeend < rangestart)
 					goto improper;
 				cp = ep + 1;
-
-#ifndef NO_QUAD
-				entitylen = strtoll(cp, &ep, 10);
-#else
-				entitylen = strtol(cp, &ep, 10);
-#endif
+				entitylen = STRTOLL(cp, &ep, 10);
 				if (entitylen < 0 || *ep != '\0')
 					goto improper;
 
 				if (debug)
-#ifndef NO_QUAD
 					fprintf(ttyout,
-					    "parsed range as: %lld-%lld/%lld\n",
-					    (long long)rangestart,
-					    (long long)rangeend,
-					    (long long)entitylen);
-#else
-					fprintf(ttyout,
-					    "parsed range as: %ld-%ld/%ld\n",
-					    (long)rangestart,
-					    (long)rangeend,
-					    (long)entitylen);
-#endif
+					    "parsed range as: "
+					    LLF "-" LLF "/" LLF "\n",
+					    (LLT)rangestart,
+					    (LLT)rangeend,
+					    (LLT)entitylen);
 				if (! restart_point) {
 					warnx(
 				    "Received unexpected Content-Range header");
@@ -1178,11 +1064,11 @@ fetch_url(const char *url, const char *proxyenv, char *proxyauth, char *wwwauth)
 			chunksize = strtol(xferbuf, &ep, 16);
 
 				/*
-				 * XXX:	Work around bug in Apache 1.3.9, which
-				 *	incorrectly puts a trailing space after
-				 *	the chunksize.
+				 * XXX:	Work around bug in Apache 1.3.9 and
+				 *	1.3.11, which incorrectly put trailing
+				 *	space after the chunksize.
 				 */
-			if (*ep == ' ')
+			while (*ep == ' ')
 				ep++;
 
 			if (strcmp(ep, "\r\n") != 0) {
@@ -1190,14 +1076,8 @@ fetch_url(const char *url, const char *proxyenv, char *proxyauth, char *wwwauth)
 				goto cleanup_fetch_url;
 			}
 			if (debug)
-				fprintf(ttyout,
-#ifndef NO_QUAD
-				    "got chunksize of %lld\n",
-				    (long long)chunksize);
-#else
-				    "got chunksize of %ld\n",
-				    (long)chunksize);
-#endif
+				fprintf(ttyout, "got chunksize of " LLF "\n",
+				    (LLT)chunksize);
 			if (chunksize == 0)
 				break;
 		}
@@ -1292,10 +1172,10 @@ fetch_url(const char *url, const char *proxyenv, char *proxyauth, char *wwwauth)
 	rval = 0;
 	goto cleanup_fetch_url;
 
-improper:
+ improper:
 	warnx("Improper response from `%s'", host);
 
-cleanup_fetch_url:
+ cleanup_fetch_url:
 	if (oldintr)
 		(void)xsignal(SIGINT, oldintr);
 	if (oldintp)
@@ -1493,10 +1373,10 @@ fetch_ftp(const char *url)
 
 	switch (type) {
 	case TYPE_A:
-		setascii(0, NULL);
+		setascii(1, xargv);
 		break;
 	case TYPE_I:
-		setbinary(0, NULL);
+		setbinary(1, xargv);
 		break;
 	default:
 		errx(1, "fetch_ftp: unknown transfer type %d", type);
@@ -1652,7 +1532,7 @@ fetch_ftp(const char *url)
 	if ((code / 100) == COMPLETE)
 		rval = 0;
 
-cleanup_fetch_ftp:
+ cleanup_fetch_ftp:
 	FREEPTR(host);
 	FREEPTR(path);
 	FREEPTR(user);
@@ -1830,7 +1710,7 @@ auto_put(int argc, char **argv, const char *uploadserver)
 		}
 	}
 	if (debug)
-		fprintf(ttyout, "autoput: url `%s' argv[2] `%s'\n",
+		fprintf(ttyout, "auto_put: url `%s' argv[2] `%s'\n",
 		    path, uargv[2] ? uargv[2] : "<null>");
 		
 			/* connect and cwd */		 
@@ -1852,7 +1732,7 @@ auto_put(int argc, char **argv, const char *uploadserver)
 	}
 	rval = 0;
 
-cleanup_auto_put:
+ cleanup_auto_put:
 	FREEPTR(uargv[2]);
 	return (rval);
 }
