@@ -1,4 +1,4 @@
-/*	$NetBSD: ftp.c,v 1.100 2000/06/11 15:15:52 lukem Exp $	*/
+/*	$NetBSD: ftp.c,v 1.109 2000/09/28 12:29:24 lukem Exp $	*/
 
 /*-
  * Copyright (c) 1996-2000 The NetBSD Foundation, Inc.
@@ -113,65 +113,48 @@ char	pasv[BUFSIZ];	/* passive port for proxy data connection */
 
 static int empty(FILE *, FILE *, int);
 
-union sockunion {
-	struct sockinet {
-#ifdef BSD4_4
-		u_char si_len;
-		u_char si_family;
-#else
-		u_short si_family;
-#endif
-		u_short si_port;
-#ifndef BSD4_4
-		u_char  si_pad[
+struct sockinet {
+	union sockunion {
+		struct sockaddr_in  su_sin;
 #ifdef INET6
-				sizeof(struct sockaddr_in6)
-#else
-				sizeof(struct sockaddr_in)
+		struct sockaddr_in6 su_sin6;
 #endif
-				- sizeof(u_int)];
-		u_char	si_len;
-#endif
-	} su_si;
-	struct sockaddr_in  su_sin;
-#ifdef INET6
-	struct sockaddr_in6 su_sin6;
+	} si_su;
+#if !HAVE_SOCKADDR_SA_LEN
+	int	si_len;
 #endif
 };
 
-#define	su_len		su_si.si_len
-#define	su_family	su_si.si_family
-#define	su_port		su_si.si_port
+#if !HAVE_SOCKADDR_SA_LEN
+# define su_len		si_len
+#else
+# define su_len		si_su.su_sin.sin_len
+#endif
+#define su_family	si_su.su_sin.sin_family
+#define su_port		si_su.su_sin.sin_port
 
-union sockunion myctladdr, hisctladdr, data_addr;
+struct sockinet myctladdr, hisctladdr, data_addr;
 
 char *
 hookup(char *host, char *port)
 {
-	int s = -1, len, error;
-#if defined(NI_NUMERICHOST) && defined(INET6)
+	int s = -1, len, error, portnum;
 	struct addrinfo hints, *res, *res0;
 	char hbuf[MAXHOSTNAMELEN];
-#else
-	struct hostent *hp = NULL;
-	char **ptr, *ep;
-	struct sockaddr_in sin;
-	long nport;
-#endif
 	static char hostnamebuf[MAXHOSTNAMELEN];
 	char *cause = "unknown";
-	int family;
-	
-#if defined(NI_NUMERICHOST) && defined(INET6)
+
 	memset((char *)&hisctladdr, 0, sizeof (hisctladdr));
+	memset((char *)&myctladdr, 0, sizeof (myctladdr));
 	memset(&hints, 0, sizeof(hints));
+	portnum = parseport(port, FTP_PORT);
 	hints.ai_flags = AI_CANONNAME;
 	hints.ai_family = AF_UNSPEC;
 	hints.ai_socktype = SOCK_STREAM;
 	hints.ai_protocol = 0;
-	error = getaddrinfo(host, port, &hints, &res0);
+	error = getaddrinfo(host, NULL, &hints, &res0);
 	if (error) {
-		warnx(gai_strerror(error));
+		warnx("%s", gai_strerror(error));
 		code = -1;
 		return (0);
 	}
@@ -210,10 +193,11 @@ hookup(char *host, char *port)
 #endif
 		{
 			getnameinfo(res->ai_addr, res->ai_addrlen,
-				hbuf, sizeof(hbuf), NULL, 0, NI_NUMERICHOST);
+			    hbuf, sizeof(hbuf), NULL, 0, NI_NUMERICHOST);
 			fprintf(ttyout, "Trying %s...\n", hbuf);
 		}
-		s = socket(res->ai_family, res->ai_socktype, res->ai_protocol);
+		((struct sockaddr_in *)res->ai_addr)->sin_port = htons(portnum);
+		s = socket(res->ai_family, SOCK_STREAM, res->ai_protocol);
 		if (s < 0) {
 			cause = "socket";
 			continue;
@@ -226,8 +210,8 @@ hookup(char *host, char *port)
 			/* this "if" clause is to prevent print warning twice */
 			if (res->ai_next) {
 				getnameinfo(res->ai_addr, res->ai_addrlen,
-					hbuf, sizeof(hbuf), NULL, 0,
-					NI_NUMERICHOST);
+				    hbuf, sizeof(hbuf), NULL, 0,
+				    NI_NUMERICHOST);
 				warn("connect to address %s", hbuf);
 			}
 			cause = "connect";
@@ -240,98 +224,26 @@ hookup(char *host, char *port)
 		break;
 	}
 	if (s < 0) {
-		warn(cause);
+		warn("%s", cause);
 		code = -1;
 		freeaddrinfo(res0);
 		return 0;
 	}
-	memcpy(&hisctladdr, res->ai_addr, res->ai_addrlen);
-	len = res->ai_addrlen;
+	memcpy(&hisctladdr.si_su, res->ai_addr, res->ai_addrlen);
+	hisctladdr.su_len = res->ai_addrlen;
 	freeaddrinfo(res0);
 	res0 = res = NULL;
-	family = hisctladdr.su_family;
-#else
-	memset(&sin, 0, sizeof(sin));
-	sin.sin_family = AF_INET;
-	if ((hp = gethostbyname(host)) == NULL) {
-		warnx("%s: %s", host, hstrerror(h_errno));
-		code = -1;
-		return 0;
-	}
 
-	nport = strtol(port, &ep, 10);
-	if (*ep != '\0' && ep == port) {
-		struct servent	*svp;
-
-		svp = getservbyname(port, "tcp");
-		if (svp == NULL) {
-			warnx("hookup: unknown port `%s'", port);
-			sin.sin_port = htons(FTP_PORT);
-		} else
-			sin.sin_port = svp->s_port;
-	} else if (nport < 1 || nport > MAX_IN_PORT_T || *ep != '\0') {
-		warnx("hookup: invalid port `%s'", port);
-		sin.sin_port = htons(FTP_PORT);
-	} else
-		sin.sin_port = htons(nport);
-
-	(void)strlcpy(hostnamebuf, hp->h_name, sizeof(hostnamebuf));
-	hostname = hostnamebuf;
-
-	if (hp->h_length > sizeof(sin.sin_addr))
-		hp->h_length = sizeof(sin.sin_addr);
-
-	for (ptr = hp->h_addr_list; *ptr; ptr++) {
-		memcpy(&sin.sin_addr, *ptr, (size_t)hp->h_length);
-		if (hp->h_addr_list[1])
-			fprintf(ttyout, "Trying %s...\n",
-			    inet_ntoa(sin.sin_addr));
-		s = socket(AF_INET, SOCK_STREAM, 0);
-		if (s < 0) {
-			cause = "socket";
-			continue;
-		}
-		while ((error = xconnect(s, (struct sockaddr *)&sin,
-		    sizeof(sin))) < 0 && errno == EINTR) {
-			;
-		}
-		if (error) {
-			/* this "if" clause is to prevent print warning twice */
-			if (hp->h_addr_list[1]) {
-				warn("connect to address %s",
-				    inet_ntoa(sin.sin_addr));
-			}
-			cause = "connect";
-			close(s);
-			s = -1;
-			continue;
-		}
-
-		/* finally we got one */
-		break;
-	}
-	if (s < 0) {
-		warn(cause);
-		code = -1;
-		return 0;
-	}
-	memcpy(&hisctladdr, &sin, sizeof(sin));
-	len = sizeof(sin);
-	if (hisctladdr.su_len == 0)
-		hisctladdr.su_len = len;
-	family = AF_INET;
-#endif
-
-	if (getsockname(s, (struct sockaddr *)&myctladdr, &len) < 0) {
+	len = hisctladdr.su_len;
+	if (getsockname(s, (struct sockaddr *)&myctladdr.si_su, &len) < 0) {
 		warn("getsockname");
 		code = -1;
 		goto bad;
 	}
-	if (myctladdr.su_len == 0)
-		myctladdr.su_len = len;
+	myctladdr.su_len = len;
 
 #ifdef IPTOS_LOWDELAY
-	if (family == AF_INET) {
+	if (hisctladdr.su_family == AF_INET) {
 		int tos = IPTOS_LOWDELAY;
 		if (setsockopt(s, IPPROTO_IP, IP_TOS, (char *)&tos,
 			       sizeof(int)) < 0)
@@ -369,7 +281,7 @@ hookup(char *host, char *port)
 	}
 
 	return (hostname);
-bad:
+ bad:
 	(void)close(s);
 	return (NULL);
 }
@@ -402,7 +314,6 @@ cmdtimeout(int notused)
 	errno = oerrno;
 }
 
-
 /*VARARGS*/
 int
 command(const char *fmt, ...)
@@ -566,20 +477,19 @@ getreply(int expecteof)
 			(void)putc(c, ttyout);
 			(void)fflush (ttyout);
 		}
-		if (line == 0) {
-			size_t len = cp - current_line;
-
-			if (len > sizeof(reply_string))
-				len = sizeof(reply_string);
-
-			(void)strlcpy(reply_string, current_line, len);
-		}
+		if (cp[-1] == '\r')
+			cp[-1] = '\0';
+		*cp = '\0';
+		if (line == 0)
+			(void)strlcpy(reply_string, current_line,
+			    sizeof(reply_string));
+		if (line > 0 && code == 0 && reply_callback != NULL)
+			(*reply_callback)(current_line);
 		if (continuation && code != originalcode) {
 			if (originalcode == 0)
 				originalcode = code;
 			continue;
 		}
-		*cp = '\0';
 		if (n != '1')
 			cpend = 0;
 		alarmtimer(0);
@@ -602,7 +512,7 @@ empty(FILE *cin, FILE *din, int sec)
 	int nr;
 	int nfd = 0;
 
-#ifdef __USE_SELECT
+#ifdef USE_SELECT
 	struct timeval t;
 	fd_set rmask;
 
@@ -790,12 +700,7 @@ sendrequest(const char *cmd, const char *local, const char *remote,
 			warn("local: %s", local);
 			goto cleanupsend;
 		}
-#ifndef NO_QUAD
-		if (command("REST %lld", (long long) restart_point) !=
-#else
-		if (command("REST %ld", (long) restart_point) !=
-#endif
-		    CONTINUE)
+		if (command("REST " LLF, (LLT)restart_point) != CONTINUE)
 			goto cleanupsend;
 		lmode = "r+w";
 	}
@@ -862,7 +767,7 @@ sendrequest(const char *cmd, const char *local, const char *remote,
 					usleep(1000000 - td.tv_usec);
 				}
 			}
-		} else {		/* simpler/faster no rate limit */
+		} else {		/* simpler/faster; no rate limit */
 			while (1) {
 				errno = c = d = 0;
 				if ((c = read(fileno(fin), buf, bufsize)) <= 0)
@@ -948,7 +853,7 @@ sendrequest(const char *cmd, const char *local, const char *remote,
 		ptransfer(0);
 	goto cleanupsend;
 
-abort:
+ abort:
 	(void)xsignal(SIGINT, oldintr);
 	oldintr = NULL;
 	if (!cpend) {
@@ -968,7 +873,7 @@ abort:
 	if (bytes > 0)
 		ptransfer(0);
 
-cleanupsend:
+ cleanupsend:
 	if (oldintr)
 		(void)xsignal(SIGINT, oldintr);
 	if (oldintp)
@@ -1099,11 +1004,7 @@ recvrequest(const char *cmd, const char *local, const char *remote,
 	if (sigsetjmp(xferabort, 1))
 		goto abort;
 	if (is_retr && restart_point &&
-#ifndef NO_QUAD
-	    command("REST %lld", (long long) restart_point) != CONTINUE)
-#else
-	    command("REST %ld", (long) restart_point) != CONTINUE)
-#endif
+	    command("REST " LLF, (LLT) restart_point) != CONTINUE)
 		goto cleanuprecv;
 	if (! EMPTYSTRING(remote)) {
 		if (command("%s %s", cmd, remote) != PRELIM)
@@ -1245,7 +1146,7 @@ recvrequest(const char *cmd, const char *local, const char *remote,
 					i++;
 			}
 			if (fseek(fout, 0L, SEEK_CUR) < 0) {
-done:
+ done:
 				warn("local: %s", local);
 				goto cleanuprecv;
 			}
@@ -1326,7 +1227,7 @@ break2:
 	}
 	goto cleanuprecv;
 
-abort:
+ abort:
 			/*
 			 * abort using RFC 959 recommended IP,SYNC sequence
 			 */
@@ -1343,7 +1244,7 @@ abort:
 	if (bytes > 0)
 		ptransfer(0);
 
-cleanuprecv:
+ cleanuprecv:
 	if (oldintr)
 		(void)xsignal(SIGINT, oldintr);
 	if (oldintp)
@@ -1378,12 +1279,12 @@ initconn(void)
 
 #ifdef INET6
 	if (myctladdr.su_family == AF_INET6 && debug &&
-	    (IN6_IS_ADDR_LINKLOCAL(&myctladdr.su_sin6.sin6_addr) ||
-	     IN6_IS_ADDR_SITELOCAL(&myctladdr.su_sin6.sin6_addr))) {
+	    (IN6_IS_ADDR_LINKLOCAL(&myctladdr.si_su.su_sin6.sin6_addr) ||
+	     IN6_IS_ADDR_SITELOCAL(&myctladdr.si_su.su_sin6.sin6_addr))) {
 		warnx("use of scoped address can be troublesome");
 	}
 #endif
-reinit:
+ reinit:
 	if (passivemode) {
 		data_addr = myctladdr;
 		data = socket(data_addr.su_family, SOCK_STREAM, 0);
@@ -1476,7 +1377,7 @@ reinit:
 		if (strcmp(pasvcmd, "PASV") == 0) {
 			if (data_addr.su_family != AF_INET) {
 				fputs(
-"Passive mode AF mismatch. Shouldn't happen!\n", ttyout);
+    "Passive mode AF mismatch. Shouldn't happen!\n", ttyout);
 				error = 1;
 				goto bad;
 			}
@@ -1499,8 +1400,8 @@ reinit:
 			memset(&data_addr, 0, sizeof(data_addr));
 			data_addr.su_family = AF_INET;
 			data_addr.su_len = sizeof(struct sockaddr_in);
-			data_addr.su_sin.sin_addr.s_addr =
-				htonl(pack4(addr, 0));
+			data_addr.si_su.su_sin.sin_addr.s_addr =
+			    htonl(pack4(addr, 0));
 			data_addr.su_port = htons(pack2(port, 0));
 		} else if (strcmp(pasvcmd, "LPSV") == 0) {
 			if (code / 10 == 22 && code != 228) {
@@ -1533,8 +1434,8 @@ reinit:
 				memset(&data_addr, 0, sizeof(data_addr));
 				data_addr.su_family = AF_INET;
 				data_addr.su_len = sizeof(struct sockaddr_in);
-				data_addr.su_sin.sin_addr.s_addr =
-					htonl(pack4(addr, 0));
+				data_addr.si_su.su_sin.sin_addr.s_addr =
+				    htonl(pack4(addr, 0));
 				data_addr.su_port = htons(pack2(port, 0));
 				break;
 #ifdef INET6
@@ -1568,7 +1469,7 @@ reinit:
 			    {
 				int i;
 				for (i = 0; i < sizeof(struct in6_addr); i++) {
-					data_addr.su_sin6.sin6_addr.s6_addr[i] =
+					data_addr.si_su.su_sin6.sin6_addr.s6_addr[i] =
 					    UC(addr[i]);
 				}
 			    }
@@ -1606,7 +1507,7 @@ reinit:
 		} else
 			goto bad;
 
-		while (xconnect(data, (struct sockaddr *)&data_addr,
+		while (xconnect(data, (struct sockaddr *)&data_addr.si_su,
 			    data_addr.su_len) < 0) {
 			if (errno == EINTR)
 				continue;
@@ -1633,7 +1534,7 @@ reinit:
 		return (0);
 	}
 
-noport:
+ noport:
 	data_addr = myctladdr;
 	if (sendport)
 		data_addr.su_port = 0;	/* let system pick one */
@@ -1652,7 +1553,8 @@ noport:
 			warn("setsockopt (reuse address)");
 			goto bad;
 		}
-	if (bind(data, (struct sockaddr *)&data_addr, data_addr.su_len) < 0) {
+	if (bind(data, (struct sockaddr *)&data_addr.si_su,
+	    data_addr.su_len) < 0) {
 		warn("bind");
 		goto bad;
 	}
@@ -1660,11 +1562,13 @@ noport:
 	    setsockopt(data, SOL_SOCKET, SO_DEBUG, (char *)&on,
 			sizeof(on)) < 0)
 		warn("setsockopt (ignored)");
-	len = sizeof(data_addr);
-	if (getsockname(data, (struct sockaddr *)&data_addr, &len) < 0) {
+	len = sizeof(data_addr.si_su);
+	memset((char *)&data_addr, 0, sizeof (data_addr));
+	if (getsockname(data, (struct sockaddr *)&data_addr.si_su, &len) < 0) {
 		warn("getsockname");
 		goto bad;
 	}
+	data_addr.su_len = len;
 	if (xlisten(data, 1) < 0)
 		warn("listen");
 
@@ -1684,9 +1588,9 @@ noport:
 #ifdef INET6
 		case AF_INET6:
 			af = (data_addr.su_family == AF_INET) ? 1 : 2;
-			if (getnameinfo((struct sockaddr *)&data_addr,
-					data_addr.su_len, hname, sizeof(hname),
-					NULL, 0, NI_NUMERICHOST)) {
+			if (getnameinfo((struct sockaddr *)&data_addr.si_su,
+			    data_addr.su_len, hname, sizeof(hname), NULL, 0,
+			    NI_NUMERICHOST)) {
 				result = ERROR;
 			} else {
 				result = command("EPRT |%d|%s|%d|", af, hname,
@@ -1712,7 +1616,7 @@ noport:
 
 		switch (data_addr.su_family) {
 		case AF_INET:
-			a = (char *)&data_addr.su_sin.sin_addr;
+			a = (char *)&data_addr.si_su.su_sin.sin_addr;
 			p = (char *)&data_addr.su_port;
 			result = command("PORT %d,%d,%d,%d,%d,%d",
 				 UC(a[0]), UC(a[1]), UC(a[2]), UC(a[3]),
@@ -1720,7 +1624,7 @@ noport:
 			break;
 #ifdef INET6
 		case AF_INET6:
-			a = (char *)&data_addr.su_sin6.sin6_addr;
+			a = (char *)&data_addr.si_su.su_sin6.sin6_addr;
 			p = (char *)&data_addr.su_port;
 			result = command(
 	"LPRT %d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d",
@@ -1757,7 +1661,7 @@ noport:
 	}
 #endif
 	return (0);
-bad:
+ bad:
 	(void)close(data), data = -1;
 	if (tmpno)
 		sendport = 1;
@@ -1767,13 +1671,13 @@ bad:
 FILE *
 dataconn(const char *lmode)
 {
-	union sockunion from;
+	struct sockinet from;
 	int s, fromlen = myctladdr.su_len;
 
 	if (passivemode)
 		return (fdopen(data, lmode));
 
-	s = accept(data, (struct sockaddr *) &from, &fromlen);
+	s = accept(data, (struct sockaddr *) &from.si_su, &fromlen);
 	if (s < 0) {
 		warn("accept");
 		(void)close(data), data = -1;
@@ -1810,8 +1714,8 @@ pswitch(int flag)
 	static struct comvars {
 		int connect;
 		char name[MAXHOSTNAMELEN];
-		union sockunion mctl;
-		union sockunion hctl;
+		struct sockinet mctl;
+		struct sockinet hctl;
 		FILE *in;
 		FILE *out;
 		int tpe;
@@ -1952,12 +1856,8 @@ proxtrans(const char *cmd, const char *local, const char *remote)
 		goto abort;
 	oldintr = xsignal(SIGINT, abortpt);
 	if ((restart_point &&
-#ifndef NO_QUAD
-	    (command("REST %lld", (long long) restart_point) != CONTINUE)
-#else
-	    (command("REST %ld", (long) restart_point) != CONTINUE)
-#endif
-	    ) || (command("%s %s", cmd, remote) != PRELIM)) {
+	    (command("REST " LLF, (LLT) restart_point) != CONTINUE))
+	    || (command("%s %s", cmd, remote) != PRELIM)) {
 		(void)xsignal(SIGINT, oldintr);
 		pswitch(1);
 		return;
@@ -1966,12 +1866,8 @@ proxtrans(const char *cmd, const char *local, const char *remote)
 	pswitch(1);
 	secndflag++;
 	if ((restart_point &&
-#ifndef NO_QUAD
-	    (command("REST %lld", (long long) restart_point) != CONTINUE)
-#else
-	    (command("REST %ld", (long) restart_point) != CONTINUE)
-#endif
-	    ) || (command("%s %s", cmd2, local) != PRELIM))
+	    (command("REST " LLF, (LLT) restart_point) != CONTINUE))
+	    || (command("%s %s", cmd2, local) != PRELIM))
 		goto abort;
 	ptflag++;
 	(void)getreply(0);
@@ -1982,7 +1878,7 @@ proxtrans(const char *cmd, const char *local, const char *remote)
 	ptflag = 0;
 	fprintf(ttyout, "local: %s remote: %s\n", local, remote);
 	return;
-abort:
+ abort:
 	if (sigsetjmp(xferabort, 1)) {
 		(void)xsignal(SIGINT, oldintr);
 		return;
@@ -2195,7 +2091,7 @@ ai_unmapped(struct addrinfo *ai)
 	sin.sin_port = sin6->sin6_port;
 
 	ai->ai_family = AF_INET;
-#ifdef BSD4_4
+#if HAVE_SOCKADDR_SA_LEN
 	sin.sin_len = len;
 #endif
 	memcpy(ai->ai_addr, &sin, len);
