@@ -1,4 +1,4 @@
-/*	$NetBSD: main.c,v 1.72 2000/06/11 02:12:06 lukem Exp $	*/
+/*	$NetBSD: main.c,v 1.73 2000/07/18 07:16:56 lukem Exp $	*/
 
 /*-
  * Copyright (c) 1996-2000 The NetBSD Foundation, Inc.
@@ -170,6 +170,7 @@ main(int argc, char *argv[])
 	epsv4bad = 0;
 	upload_path = NULL;
 	isupload = 0;
+	reply_callback = NULL;
 
 	/*
 	 * Get the default socket buffer sizes if we don't already have them.
@@ -349,7 +350,6 @@ main(int argc, char *argv[])
 
 		case 'u':
 		{
-			/* XXX : if i move this under 'T' we get a segv. */
 			isupload = 1;
 			interactive = 0;
 			upload_path = xstrdup(optarg);
@@ -567,7 +567,7 @@ cmdscanner(void)
 			if (fgets(line, sizeof(line), stdin) == NULL) {
 				if (fromatty)
 					putc('\n', ttyout);
-				quit(0, 0);
+				quit(0, NULL);
 			}
 			num = strlen(line);
 			if (num == 0)
@@ -591,7 +591,7 @@ cmdscanner(void)
 			if ((buf = el_gets(el, &num)) == NULL || num == 0) {
 				if (fromatty)
 					putc('\n', ttyout);
-				quit(0, 0);
+				quit(0, NULL);
 			}
 			if (buf[--num] == '\n') {
 				if (num == 0)
