@@ -1,4 +1,4 @@
-/*	$NetBSD: util.c,v 1.96 2000/06/15 13:08:27 lukem Exp $	*/
+/*	$NetBSD: util.c,v 1.102 2000/09/08 11:54:53 lukem Exp $	*/
 
 /*-
  * Copyright (c) 1997-2000 The NetBSD Foundation, Inc.
@@ -147,64 +147,103 @@ setpeer(int argc, char *argv[])
 	}
 }
 
+static void
+parse_feat(const char *line)
+{
+
+	if (strcasecmp(line, " MDTM") == 0)
+		features[FEAT_MDTM] = 1;
+	else if (strncasecmp(line, " MLST", sizeof(" MLST") - 1) == 0) {
+		features[FEAT_MLST] = 1;
+	} else if (strcasecmp(line, " REST STREAM") == 0)
+		features[FEAT_REST_STREAM] = 1;
+	else if (strcasecmp(line, " SIZE") == 0)
+		features[FEAT_SIZE] = 1;
+	else if (strcasecmp(line, " TVFS") == 0)
+		features[FEAT_TVFS] = 1;
+}
+
 /*
- * Determine the remote system type.
+ * Determine the remote system type (SYST) and features (FEAT).
  * Call after a successful login (i.e, connected = -1)
  */
 void
-remotesyst(void)
+getremoteinfo(void)
 {
-	int overbose;
+	int overbose, i;
 
 	overbose = verbose;
 	if (debug == 0)
 		verbose = -1;
-	if (command("SYST") == COMPLETE && overbose) {
-		char *cp, c;
-		c = 0;
-		cp = strchr(reply_string + 4, ' ');
-		if (cp == NULL)
-			cp = strchr(reply_string + 4, '\r');
-		if (cp) {
-			if (cp[-1] == '.')
-				cp--;
-			c = *cp;
-			*cp = '\0';
-		}
 
-		fprintf(ttyout, "Remote system type is %s.\n",
-		    reply_string + 4);
-		if (cp)
-			*cp = c;
-	}
-	if (!strncmp(reply_string, "215 UNIX Type: L8", 17)) {
-		if (proxy)
-			unix_proxy = 1;
-		else
-			unix_server = 1;
-		/*
-		 * Set type to 0 (not specified by user),
-		 * meaning binary by default, but don't bother
-		 * telling server.  We can use binary
-		 * for text files unless changed by the user.
-		 */
-		type = 0;
-		(void)strlcpy(typename, "binary", sizeof(typename));
-		if (overbose)
-		    fprintf(ttyout,
-			"Using %s mode to transfer files.\n",
-			typename);
-	} else {
-		if (proxy)
-			unix_proxy = 0;
-		else
-			unix_server = 0;
-		if (overbose &&
-		    !strncmp(reply_string, "215 TOPS20", 10))
-			fputs(
+			/* determine remote system type */
+	if (command("SYST") == COMPLETE) {
+		if (overbose) {
+			char *cp, c;
+
+			c = 0;
+			cp = strchr(reply_string + 4, ' ');
+			if (cp == NULL)
+				cp = strchr(reply_string + 4, '\r');
+			if (cp) {
+				if (cp[-1] == '.')
+					cp--;
+				c = *cp;
+				*cp = '\0';
+			}
+
+			fprintf(ttyout, "Remote system type is %s.\n",
+			    reply_string + 4);
+			if (cp)
+				*cp = c;
+		}
+		if (!strncmp(reply_string, "215 UNIX Type: L8", 17)) {
+			if (proxy)
+				unix_proxy = 1;
+			else
+				unix_server = 1;
+			/*
+			 * Set type to 0 (not specified by user),
+			 * meaning binary by default, but don't bother
+			 * telling server.  We can use binary
+			 * for text files unless changed by the user.
+			 */
+			type = 0;
+			(void)strlcpy(typename, "binary", sizeof(typename));
+			if (overbose)
+			    fprintf(ttyout,
+				"Using %s mode to transfer files.\n",
+				typename);
+		} else {
+			if (proxy)
+				unix_proxy = 0;
+			else
+				unix_server = 0;
+			if (overbose &&
+			    !strncmp(reply_string, "215 TOPS20", 10))
+				fputs(
 "Remember to set tenex mode when transferring binary files from this machine.\n",
-			    ttyout);
+				    ttyout);
+		}
 	}
+
+			/* determine features (if any) */
+	for (i = 0; i < FEAT_max; i++)
+		features[i] = -1;
+	reply_callback = parse_feat;
+	if (command("FEAT") == COMPLETE) {
+		for (i = 0; i < FEAT_max; i++) {
+			if (features[i] == -1)
+				features[i] = 0;
+		}
+		features[FEAT_FEAT] = 1;
+	} else
+		features[FEAT_FEAT] = 0;
+	if (debug)
+		for (i = 0; i < FEAT_max; i++)
+			printf("features[%d] = %d\n", i, features[i]);
+	reply_callback = NULL;
+
 	verbose = overbose;
 }
 
@@ -399,7 +438,7 @@ ftp_login(const char *host, const char *user, const char *pass)
 		goto cleanup_ftp_login;
 
 	connected = -1;
-	remotesyst();
+	getremoteinfo();
 	for (n = 0; n < macnum; ++n) {
 		if (!strcmp("init", macros[n].mac_name)) {
 			(void)strlcpy(line, "$init", sizeof(line));
@@ -410,7 +449,7 @@ ftp_login(const char *host, const char *user, const char *pass)
 	}
 	updateremotepwd();
 
-cleanup_ftp_login:
+ cleanup_ftp_login:
 	if (user != NULL && freeuser)
 		free((char *)user);
 	if (pass != NULL && freepass)
@@ -568,31 +607,39 @@ globulize(const char *pattern)
 off_t
 remotesize(const char *file, int noisy)
 {
-	int overbose;
+	int overbose, r;
 	off_t size;
 
 	overbose = verbose;
 	size = -1;
 	if (debug == 0)
 		verbose = -1;
-	if (command("SIZE %s", file) == COMPLETE) {
+	if (! features[FEAT_SIZE]) {
+		if (noisy)
+			fprintf(ttyout,
+			    "SIZE is not supported by remote server.\n");
+		goto cleanup_remotesize;
+	}
+	r = command("SIZE %s", file);
+	if (r == COMPLETE) {
 		char *cp, *ep;
 
 		cp = strchr(reply_string, ' ');
 		if (cp != NULL) {
 			cp++;
-#ifndef NO_QUAD
-			size = strtoll(cp, &ep, 10);
-#else
-			size = strtol(cp, &ep, 10);
-#endif
+			size = STRTOLL(cp, &ep, 10);
 			if (*ep != '\0' && !isspace((unsigned char)*ep))
 				size = -1;
 		}
-	} else if (noisy && debug == 0) {
-		fputs(reply_string, ttyout);
-		putc('\n', ttyout);
+	} else {
+		if (r == ERROR && code == 500 && features[FEAT_SIZE] == -1)
+			features[FEAT_SIZE] = 0;
+		if (noisy && debug == 0) {
+			fputs(reply_string, ttyout);
+			putc('\n', ttyout);
+		}
 	}
+ cleanup_remotesize:
 	verbose = overbose;
 	return (size);
 }
@@ -603,16 +650,22 @@ remotesize(const char *file, int noisy)
 time_t
 remotemodtime(const char *file, int noisy)
 {
-	int overbose;
-	time_t rtime;
-	int ocode;
+	int	overbose, ocode, r;
+	time_t	rtime;
 
 	overbose = verbose;
 	ocode = code;
 	rtime = -1;
 	if (debug == 0)
 		verbose = -1;
-	if (command("MDTM %s", file) == COMPLETE) {
+	if (! features[FEAT_MDTM]) {
+		if (noisy)
+			fprintf(ttyout,
+			    "MDTM is not supported by remote server.\n");
+		goto cleanup_parse_time;
+	}
+	r = command("MDTM %s", file);
+	if (r == COMPLETE) {
 		struct tm timebuf;
 		char *timestr, *frac;
 		int yy, mo, day, hour, min, sec;
@@ -670,9 +723,13 @@ remotemodtime(const char *file, int noisy)
 				goto cleanup_parse_time;
 		} else if (debug)
 			fprintf(ttyout, "parsed date as: %s", ctime(&rtime));
-	} else if (noisy && debug == 0) {
-		fputs(reply_string, ttyout);
-		putc('\n', ttyout);
+	} else {
+		if (r == ERROR && code == 500 && features[FEAT_MDTM] == -1)
+			features[FEAT_MDTM] = 0;
+		if (noisy && debug == 0) {
+			fputs(reply_string, ttyout);
+			putc('\n', ttyout);
+		}
 	}
  cleanup_parse_time:
 	verbose = overbose;
@@ -819,6 +876,12 @@ progressmeter(int flag)
 	if (!progress || filesize <= 0)
 		return;
 
+	/*
+	 * print progress bar only if we are foreground process.
+	 */
+	if (! foregroundproc())
+		return;
+
 	(void)gettimeofday(&now, NULL);
 	cursize = bytes + restart_point;
 	timersub(&now, &lastupdate, &wait);
@@ -828,12 +891,6 @@ progressmeter(int flag)
 		wait.tv_sec = 0;
 	}
 
-	/*
-	 * print progress bar only if we are foreground process.
-	 */
-	if (! foregroundproc())
-		return;
-
 	ratio = (int)((double)cursize * 100.0 / (double)filesize);
 	ratio = MAX(ratio, 0);
 	ratio = MIN(ratio, 100);
@@ -853,12 +910,8 @@ progressmeter(int flag)
 	abbrevsize = cursize;
 	for (i = 0; abbrevsize >= 100000 && i < sizeof(prefixes); i++)
 		abbrevsize >>= 10;
-	len += snprintf(buf + len, BUFLEFT,
-#ifndef NO_QUAD
-	    " %5lld %c%c ", (long long)abbrevsize,
-#else
-	    " %5ld %c%c ", (long)abbrevsize,
-#endif
+	len += snprintf(buf + len, BUFLEFT, " " LLFP("5") " %c%c ",
+	    (LLT)abbrevsize,
 	    prefixes[i],
 	    i == 0 ? ' ' : 'B');
 
@@ -874,11 +927,8 @@ progressmeter(int flag)
 	for (i = 1; bytespersec >= 1024000 && i < sizeof(prefixes); i++)
 		bytespersec >>= 10;
 	len += snprintf(buf + len, BUFLEFT,
-#ifndef NO_QUAD
-	    " %3lld.%02d %cB/s ", (long long)bytespersec / 1024,
-#else
-	    " %3ld.%02d %cB/s ", (long)bytespersec / 1024,
-#endif
+	    " " LLFP("3") ".%02d %cB/s ",
+	    (LLT)(bytespersec / 1024),
 	    (int)((bytespersec % 1024) * 100 / 1024),
 	    prefixes[i]);
 
@@ -947,13 +997,8 @@ ptransfer(int siginfo)
 			bytespersec /= elapsed;
 	}
 	len = 0;
-	len += snprintf(buf + len, BUFLEFT,
-#ifndef NO_QUAD
-	    "%lld byte%s %s in ", (long long)bytes,
-#else
-	    "%ld byte%s %s in ", (long)bytes,
-#endif
-	    bytes == 1 ? "" : "s", direction);
+	len += snprintf(buf + len, BUFLEFT, LLF " byte%s %s in ",
+	    (LLT)bytes, bytes == 1 ? "" : "s", direction);
 	remaining = (int)elapsed;
 	if (remaining > SECSPERDAY) {
 		int days;
@@ -972,12 +1017,8 @@ ptransfer(int siginfo)
 
 	for (i = 1; bytespersec >= 1024000 && i < sizeof(prefixes); i++)
 		bytespersec >>= 10;
-	len += snprintf(buf + len, BUFLEFT,
-#ifndef NO_QUAD
-	    "(%lld.%02d %cB/s)", (long long)bytespersec / 1024,
-#else
-	    "(%ld.%02d %cB/s)", (long)bytespersec / 1024,
-#endif
+	len += snprintf(buf + len, BUFLEFT, "(" LLF ".%02d %cB/s)",
+	    (LLT)(bytespersec / 1024),
 	    (int)((bytespersec % 1024) * 100 / 1024),
 	    prefixes[i]);
 
@@ -1023,10 +1064,10 @@ void
 list_vertical(StringList *sl)
 {
 	int i, j, w;
-	int columns, width, lines, items;
+	int columns, width, lines;
 	char *p;
 
-	width = items = 0;
+	width = 0;
 
 	for (i = 0 ; i < sl->sl_cur ; i++) {
 		w = strlen(sl->sl_str[i]);
@@ -1359,6 +1400,39 @@ formatbuf(char *buf, size_t len, const char *src)
 	buf[i] = '\0';
 }
 
+/*
+ * Parse `port' into a TCP port number, defaulting to `defport' if `port' is
+ * an unknown service name. If defport != -1, print a warning upon bad parse.
+ */
+int
+parseport(const char *port, int defport)
+{
+	int	 rv;
+	long	 nport;
+	char	*p, *ep;
+
+	p = xstrdup(port);
+	nport = strtol(p, &ep, 10);
+	if (*ep != '\0' && ep == p) {
+		struct servent	*svp;
+
+		svp = getservbyname(port, "tcp");
+		if (svp == NULL) {
+ badparseport:
+			if (defport != -1)
+				warnx("Unknown port `%s', using port %d",
+				    port, defport);
+			rv = defport;
+		} else
+			rv = ntohs(svp->s_port);
+	} else if (nport < 1 || nport > MAX_IN_PORT_T || *ep != '\0')
+		goto badparseport;
+	else
+		rv = nport;
+	free(p);
+	return (rv);
+}
+
 /*
  * Determine if given string is an IPv6 address or not.
  * Return 1 for yes, 0 for no
